#chart {
    text-align:center;
    white-space:nowrap;
    overflow:auto;
    height:300px;
}
.chart-options {
    background: none repeat scroll 0 0 #EFEFEF;
    border: 1px solid #DEDEDE;
    border-radius: 5px 5px 5px 5px;
    margin: 10px 0;
    padding: 10px 15px;
}
.domain {
  fill: none;
  stroke: #3d3d3d;
  stroke-width: 1.5;
}
